
<!DOCTYPE html>
<html ng-app="Static">
<head>
    <meta charset="UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>Learning Assemble, Step 2 - Basic Templating | James Wing</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta name="description" content="Part of a series exploring the Assemble static web site framework for Node.js" />

    

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.1.0/css/font-awesome.min.css"/>

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/css/bootstrap.min.css" />



<link rel="stylesheet" type="text/css" href="../assets/css/compiled/site.css">



<!--<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/highlight.js/8.6/styles/default.min.css">-->


<link href="../assets/prism/prism.css" rel="stylesheet" />

<!--[if lt IE 9]>
<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->

<script src="//ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js"></script>


</head>
<body id="blogpost" ng-controller="BaseController">



<header class="navbar navbar-inverse normal" role="banner">
    <div class="navbar-container">
        <div class="navbar-header">
            <button class="navbar-toggle" type="button" data-toggle="collapse" data-target=".bs-navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a href="/index.html" class="navbar-brand">James Wing</a>
        </div>
        <nav class="collapse navbar-collapse bs-navbar-collapse" role="navigation">
            <ul class="nav navbar-nav navbar-right">
                

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


            </ul>
        </nav>
    </div>
</header>



<div class="layout-sidebar">
    <div class="row">
        <div class="col-sm-12 col-md-9 col-lg-8">
            

<div class="blogpost-wrapper">
    

    <div class="row">
        <div class="col-md-10 col-lg-10 post">
            <h1 class="title">
                <a href="">Learning Assemble, Step 2 - Basic Templating</a>
            </h1>
            <!--<div class="author">-->
                <!---->
                <!---->
                <!---->
                <!--2014-10-23 by James Wing-->
            <!--</div>-->
            <div class="date">
                2014-10-23
            </div>

            

            <div class="content">

                
                    <p>Our first Assemble example was nicely minimal, but sadly not very useful or realistic.  In this post, we&#x27;ll flesh out
our Assemble templating a bit more to convert from &quot;Hello, Assemble!&quot; to something just slightly more realistic.
This will include:</p>
<ul>
<li>Designing a simple folder structure</li>
<li>Adding some more pages</li>
<li>Using layouts and partial Handlebars templates</li>
</ul>

                    <h2 id="simple-folder-structure">Simple Folder Structure</h2>
<p>The first thing we need is a more realistic folder structure that will grow to hold a modest number of page templates, layouts, static asset files, and other stuff.  We&#39;ll keep &#39;output&#39; as the root of the output file tree for now.  I recommend a &#39;source&#39; folder, roughly following some of the conventions established in Assemble boilerplate projects:</p>
<pre><code>.
|-- Gruntfile.js
|-- package.json
|-- source
|    |-- templates
|         |-- layouts
|         |    +-- default.hbs
|         |-- pages
|         |    |-- index.hbs
|         |    |-- features.hbs
|         |    |-- pricing.hbs
|         |    |-- about.hbs
|         +-- partials
|              +-- header.hbs
+-- output</code></pre>
<p>This structure includes some new files we will create below.  Our three template folders will hold Handlebars template files:</p>
<ul>
<li><strong>pages</strong> - to hold the primary content for individual pages, we&#39;ll add these below.</li>
<li><strong>partials</strong> - reusable snippets of content, useful for headers, footers, script includes, and more.</li>
<li><strong>layouts</strong> - to pull all of the above together</li>
</ul>
<p>Why &#39;source/templates&#39; and not just &#39;templates&#39;?  Our project will inevitably grow to be more than just Handlebars templates.  In the future, I expect source to have many additional folders for things like stylesheets, client-side javascript, images, etc.</p>
<h2 id="updating-gruntfile-for-structure">Updating Gruntfile for Structure</h2>
<p>Now we will modify our Gruntfile to reflect the new folder structure.</p>
<pre class="language-javascript line-numbers" data-line="7,8,9,10" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                files: [
                    {
                        expand: true,
                        cwd: 'source/templates/pages',
                        src: '**/*.hbs',
                        dest: 'output/'
                    }
                ]
            }
        }
    });
    grunt.loadNpmTasks('assemble');
};
</code></pre>

<p>We only changed the page file specification on lines 11-15.  We have asked for all of the page files to be moved to the root of the output folder, relative to their place in the source pages folder.  See <a href="http://gruntjs.com/configuring-tasks#globbing-patterns">Grunt File Globbing Patterns</a> for more on the syntax.</p>
<p>If you run <code>grunt assemble</code> at this point, you should regenerate our awesome &quot;Hello, Assemble!&quot; page.</p>
<h2 id="new-pages">New Pages</h2>
<p>We need a few more pages to make our static site a bit more realistic.  Let&#39;s add some simple pages for Features, Pricing, and About Us.  The content of these pages assumes the template we will farther below.  All of these go in our source/templates/pages directory.</p>
<h3 id="features-hbs">features.hbs</h3>
<pre><code class="language-markup">---
title: &quot;Features&quot;
---
&lt;h1&gt;{{ title }}&lt;/h1&gt;
&lt;p&gt;Our features are the best&lt;/p&gt;</code></pre>
<h3 id="pricing-hbs">pricing.hbs</h3>
<pre><code class="language-markup">---
title: &quot;Pricing&quot;
---
&lt;h1&gt;{{ title }}&lt;/h1&gt;
&lt;p&gt;Our product is totally worth $1,000&lt;/p&gt;</code></pre>
<h3 id="about-hbs">about.hbs</h3>
<pre><code class="language-markup">---
title: &quot;About Us&quot;
---
&lt;h1&gt;{{ title }}&lt;/h1&gt;
&lt;p&gt;This is the About page content&lt;/p&gt;</code></pre>
<h2 id="templates-layouts-and-partials">Templates, Layouts, and Partials</h2>
<p>So far our templating has been less than impressive. We will add a simple partial template for the header, a layout, and modify our original index.hbs page to reflect the changes.</p>
<h3 id="header-partial-header-hbs-">Header Partial (header.hbs)</h3>
<p>A partial Handlebars template can be as simple as a text include, and that&#39;s all we&#39;re going to do for now.  We&#39;ll figure out how to dynamically generate this type of navigation in a later post.  Use the following for source/templates/partials/header.hbs:</p>
<pre><code class="language-markup">&lt;div&gt;
    &lt;a href=&quot;/&quot;&gt;Home&lt;/a&gt; |
    &lt;a href=&quot;/features.html&quot;&gt;Features&lt;/a&gt; |
    &lt;a href=&quot;/pricing.html&quot;&gt;Pricing&lt;/a&gt; |
    &lt;a href=&quot;/about.html&quot;&gt;About&lt;/a&gt;
&lt;/div&gt;</code></pre>
<h3 id="layout-default-hbs-">Layout (default.hbs)</h3>
<p>The layout page will combine our various partials with our page content.  This typically means it contains the outline of an HTML page, with many includes for partials for navigation, scripts, and other components.</p>
<pre><code class="language-markup">&lt;!DOCTYPE html&gt;
&lt;html&gt;
&lt;head&gt;
    &lt;title&gt;Learning Assemble, Step 2 - Basic Templating&lt;/title&gt;
&lt;/head&gt;
&lt;body&gt;
{{&gt; header }}
{{&gt; body}}
&lt;/body&gt;
&lt;/html&gt;</code></pre>
<h3 id="home-page-index-hbs-">Home Page (index.hbs)</h3>
<p>Now let&#39;s revisit our home page template.  Where we used to have the complete HTML structure, we now only need the body content, so it looks pretty lean:</p>
<pre><code class="language-markup">---
title: &quot;Home&quot;
---
&lt;h1&gt;{{ title }}&lt;/h1&gt;
&lt;p&gt;This is the home page&lt;/p&gt;</code></pre>
<h2>Updating Gruntfile for Layouts and Partials</h2>

<p>We need a bit more Assemble configuration to use the partial template and layout we just created.  We&#39;ll modify our Gruntfile as follows:</p>
<pre class="language-javascript line-numbers" data-line="6,7,8" data-start="1"><code>
module.exports = function (grunt) {
    grunt.initConfig({
        assemble: {
            hello: {
                options: {
                    layoutdir: 'source/templates/layouts',
                    layout: 'default.hbs',
                    partials: 'source/templates/partials/**/*.hbs'
                },
                files: [
                    { expand: true, cwd: 'source/templates/pages', src: '**/*.hbs', dest: 'output/' }
                ]
            }
        }
    });
    grunt.loadNpmTasks('assemble');
};
</code></pre>

<p>We specified three options for Assemble:</p>
<ol>
<li><strong>layoutdir</strong> - the directory to search for named layouts.</li>
<li><strong>layout</strong> - the default Handlebars layout, for pages that do not specify a named layout.  All of our pages are using the default for now.</li>
<li><strong>partials</strong> - a file specification for partial Handlebars template files.</li>
</ol>
<p>Try running <code>grunt assemble</code> again.  You should now generate all four output pages, each with a complete HTML structure.</p>
<pre>
Running "assemble:hello" (assemble) task
Assembling output/about.html OK
Assembling output/features.html OK
Assembling output/index.html OK
Assembling output/pricing.html OK
&gt;&gt; 4 pages assembled.

Done, without errors.
</pre>

<p>Although we have generated some output, we&#39;ve only been looking at the output in a text editor.  In the next post, we&#39;ll get a preview feedback cycle going.</p>
<p>For the code, please see the <a href="https://github.com/jvwing/learning-assemble">learning-assemble</a> repository on GitHub.
The branch <a href="https://github.com/jvwing/learning-assemble/tree/step02-basic-templating">step02-basic-templating</a>
contains the code from this post.</p>

                

                
                    
                
                    
                
                    
                        <div class="blog-post-next">
                            Next:
                            <a href="/learning-assemble/step-3-grunt-workflow.html">
                                Learning Assemble, Step 3 - Grunt Workflow <i class="fa fa-lg fa-arrow-right"></i>
                            </a>
                        </div>
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                

            </div>

            <div class="share">
                <a href="https://twitter.com/share" class="twitter-share-button" data-text="Learning Assemble, Step 2 - Basic Templating" data-via="jvwing" data-size="large">Tweet</a>
                <script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+'://platform.twitter.com/widgets.js';fjs.parentNode.insertBefore(js,fjs);}}(document, 'script', 'twitter-wjs');</script>
            </div>

            <hr/>
            <div class="other-posts">
                <h4>You might also enjoy:</h4>
                <ul>
                    
                        <li><a href="/learning-assemble/step-1-hello-assemble.html">Learning Assemble, Step 1 - Hello, Assemble!</a></li>
                    
                        <li><a href="/learning-assemble/step-2-basic-templating.html">Learning Assemble, Step 2 - Basic Templating</a></li>
                    
                        <li><a href="/learning-assemble/step-3-grunt-workflow.html">Learning Assemble, Step 3 - Grunt Workflow</a></li>
                    
                        <li><a href="/learning-assemble/step-4-data.html">Learning Assemble, Step 4 - Data</a></li>
                    
                        <li><a href="/learning-assemble/step-5-page-navigation.html">Learning Assemble, Step 5 - Page Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-6-blogging-with-markdown.html">Learning Assemble, Step 6 - Blogging with Markdown</a></li>
                    
                        <li><a href="/learning-assemble/step-7-custom-handlebars-helpers.html">Learning Assemble, Step 7 - Custom Handlebars Helpers</a></li>
                    
                        <li><a href="/learning-assemble/step-8-debugging-assemble.html">Learning Assemble, Step 8 - Debugging Assemble</a></li>
                    
                        <li><a href="/learning-assemble/step-9-sitemap.html">Learning Assemble, Step 9 - Sitemap</a></li>
                    
                        <li><a href="/learning-assemble/step-10-pages-from-data.html">Learning Assemble, Step 10 - Pages from Data</a></li>
                    
                        <li><a href="/learning-assemble/step-11-more-navigation.html">Learning Assemble, Step 11 - More Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-12-navigation-helper.html">Learning Assemble, Step 12 - Navigation Helper</a></li>
                    
                </ul>
            </div>

            <div class="comments">
                <div id="disqus_thread"></div>
<script type="text/javascript">
    /* * * CONFIGURATION VARIABLES * * */
    var disqus_shortname = 'jameswingnet';

    /* * * DON'T EDIT BELOW THIS LINE * * */
    (function() {
        var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
        dsq.src = '//' + disqus_shortname + '.disqus.com/embed.js';
        (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
    })();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript" rel="nofollow">comments powered by Disqus.</a></noscript>
            </div>

        </div>

    </div>

</div>

        </div>
        <div class="col-md-3 col-lg-4 sidebar">
            <div class="posts-list-brief">
    <h3 class="sidebar-heading">Recent</h3>
    <ul>
        
            
        
            
                
                    <li>
                        
                            <a href="/aws/query-cloudtrail-events-with-athena.html">Query CloudTrail Events with Athena</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/nifi-internal-fields.html">Hidden Fields in Apache NiFi FlowFiles</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/first-look-at-apache-nifi.html">First Look at Apache NiFi</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/monitoring/monitoring-advice-distilled.html">Monitoring Advice Distilled</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/aws/using-codecommit-and-git-credentials.html">Using CodeCommit and GitHub Credential Helpers</a>
                        
                    </li>
                
            
        
            
        
            
        
            
        
            
        
    </ul>
</div>
            <hr/>
            <div class="ads">
    <!--<h3 class="sidebar-heading">Featured</h3>-->
    <div class="ad">
        <a href="http://prontolyzer.jameswing.net/">
            <img class="thumbnail" src="/assets/ads/prontolyzer-thumb.png" />
            <h4 class="title">Prontolyzer</h4>
            <p class="description">
                Interactive mapping and exploration of bike trip data for the 2015 Pronto Cycle Share Data Challenge.
                
            </p>
        </a>
    </div>
    <hr/>
    <div class="ad">
        <a href="/troubleshooting-chart/introducing-the-troubleshooting-chart.html">
            <img class="thumbnail" src="/assets/posts/troubleshooting-chart/troubleshootingchartannotated-thumbnail.png" />
            <h4 class="title">The Troubleshooting Chart</h4>
            <p class="description">
                Level up your troubleshooting with simple data analysis and visualizations.
                A thrill-ride for Ops types.
            </p>
        </a>
    </div>
</div>
        </div>
    </div>
</div>




<div id="footer">
    <div class="container">
        <div class="row">
            <div class="col-sm-6 copyright">
                © 2014-2017 James Wing. All Rights Reserved.
            </div>
            <div class="col-sm-6 menu">
                <ul>
                    

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


                </ul>
            </div>
        </div>
    </div>
</div>






<script src="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/js/bootstrap.min.js"></script>

<script>
    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
        (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
            m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
    var gaId = 'UA-6573';
    gaId += '6738-1';
    ga('create', gaId, 'auto');
    ga('send', 'pageview');

</script>









<script src="../assets/prism/prism.js"></script>


</body>
</html>